@@ -1,28 +1,30 @@
 import csv
 import io
 import time
+from typing import Dict, List, Type, Union
 
 import pendulum
 import singer
-
 from singer import transform
+
 from tap_zuora import apis
-from tap_zuora.client import ApiException
+from tap_zuora.client import Client
+from tap_zuora.exceptions import ApiException, FileIdNotFoundException
 
 PARTNER_ID = "salesforce"
 DEFAULT_POLL_INTERVAL = 60
-DEFAULT_JOB_TIMEOUT = 5400
+DEFAULT_JOB_TIMEOUT = 12 * 60 * 60  # 12 hrs in seconds
 MAX_EXPORT_DAYS = 30
 
 LOGGER = singer.get_logger()
 
 
 def parse_csv_line(line):
-    reader = csv.reader(io.StringIO(line.decode('utf-8').replace('\0', '')))
+    reader = csv.reader(io.StringIO(line.decode("utf-8").replace("\0", "")))
     return next(reader)
 
 
-def convert_header(header, stream):
+def convert_header(header: str, stream: str) -> str:
     dotted_field = header.split(".")
     if stream == dotted_field[0]:
         return dotted_field[1]
@@ -30,11 +32,11 @@ def convert_header(header, stream):
     return header.replace(".", "")
 
 
-def parse_header_line(line, stream):
+def parse_header_line(line, stream: str) -> List:
     return [convert_header(h, stream) for h in parse_csv_line(line)]
 
 
-def poll_job_until_done(job_id, client, api):
+def poll_job_until_done(job_id: str, client: Client, api: Union[Type[apis.Rest], Type[apis.Aqua]]) -> List:
     timeout_time = pendulum.utcnow().add(seconds=DEFAULT_JOB_TIMEOUT)
     while pendulum.utcnow() < timeout_time:
         if api.job_ready(client, job_id):
@@ -44,18 +46,22 @@ def poll_job_until_done(job_id, client, api):
 
     raise apis.ExportTimedOut(DEFAULT_JOB_TIMEOUT // 60, "minutes")
 
-def clear_file_ids(state, stream):
+
+def clear_file_ids(state: Dict, stream: Dict) -> Dict:
     state["bookmarks"][stream["tap_stream_id"]].pop("file_ids", None)
     singer.write_state(state)
     return state
 
 
-def clear_stateful_session(state, stream):
+def clear_stateful_session(state: Dict, stream: Dict) -> Dict:
     state["bookmarks"][stream["tap_stream_id"]]["version"] = int(time.time())
     singer.write_state(state)
     return state
 
-def sync_file_ids(file_ids, client, state, stream, api, counter): # pylint: disable=too-many-branches
+
+def sync_file_ids(
+    file_ids: List, client: Client, state: Dict, stream: Dict, api, counter
+):  # pylint: disable=too-many-branches
     if stream.get("replication_key"):
         start_date = state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]]
     else:
@@ -74,10 +80,12 @@ def sync_file_ids(file_ids, client, state, stream, api, counter): # pylint: disa
             # Don't advance the bookmark until all files in the window have been synced.
             if ex.resp.status_code == 404:
                 clear_file_ids(state, stream)
-                raise Exception(("File ID {} has been deleted, making the sync window invalid. "
-                                 "Removing partially exported files from state and will resume "
-                                 "from bookmark on the next extraction.")
-                                .format(file_id)) from ex
+                raise FileIdNotFoundException(
+                    f"File ID {file_id} has been deleted, making the sync window invalid. "
+                    f"Removing partially exported files from state and will resume from "
+                    f"bookmark on the next extraction."
+                ) from ex
+
             raise
         header = parse_header_line(next(lines), stream["tap_stream_id"])
         extraction_time = singer.utils.now()
@@ -89,26 +97,24 @@ def sync_file_ids(file_ids, client, state, stream, api, counter): # pylint: disa
             if len(header) != len(parsed_line):
                 state = clear_file_ids(state, stream)
                 state = clear_stateful_session(state, stream)
-                raise Exception(("Detected that File ID {} is non-rectangular. Found row "
-                                 "with {} entries, expected {} entries from header line. "
-                                 "Will resume from bookmark with new AQuA session on next extraction.")
-                                .format(file_id, len(parsed_line), len(header)))
+                raise Exception(
+                    f"Detected that File ID {file_id} is non-rectangular. Found row with {len(parsed_line)} "
+                    f"entries, expected {len(header)} entries from header line. "
+                    f"Will resume from bookmark with new AQuA session on next extraction."
+                )
 
             row = dict(zip(header, parsed_line))
-            record = transform(row, stream['schema'])
+            record = transform(row, stream["schema"])
             # safe get because not all records will have 'Deleted'
-            if record.get('Deleted', False):
+            if record.get("Deleted", False):
                 # We should emit that we saw a deleted record
                 saw_deleted = True
             if stream.get("replication_key"):
                 bookmark = record.get(stream["replication_key"])
-                if not bookmark:
+                if not bookmark or bookmark < start_date:
                     # There's a chance we get back a bad record here, and we don't want to null the bookmark
                     continue
 
-                if bookmark and bookmark < start_date:
-                    continue
-
                 singer.write_record(stream["tap_stream_id"], record, time_extracted=extraction_time)
                 state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]] = bookmark
                 singer.write_state(state)
@@ -128,24 +134,30 @@ def sync_file_ids(file_ids, client, state, stream, api, counter): # pylint: disa
     singer.write_state(state)
     return counter
 
-def handle_aqua_timeout(ex, stream, state):
-    if stream.get("replication_key"):
-        LOGGER.info("Export timed out, reducing query window and writing state.")
-        window_bookmark = state["bookmarks"][stream["tap_stream_id"]].get("current_window_end")
-        previous_window_end = pendulum.parse(window_bookmark) if window_bookmark else pendulum.utcnow()
-        window_start = pendulum.parse(state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]])
-        if previous_window_end == window_start:
-            raise apis.ExportFailed("Export too large for smallest possible query window. " +
-                                    "Cannot subdivide any further. ({}: {})"
-                                    .format(stream["replication_key"], window_start)) from ex
 
-        half_day_range = (previous_window_end - window_start) // 2
-        current_window_end = previous_window_end - half_day_range
-        state["bookmarks"][stream["tap_stream_id"]]["current_window_end"] = current_window_end.strftime("%Y-%m-%dT%H:%M:%SZ")
-        singer.write_state(state)
+def handle_aqua_timeout(ex: apis.ExportTimedOut, stream: Dict, state: Dict):
+    if not stream.get("replication_key"):
+        return
+    LOGGER.info("Export timed out, reducing query window and writing state.")
+    window_bookmark = state["bookmarks"][stream["tap_stream_id"]].get("current_window_end")
+    previous_window_end = pendulum.parse(window_bookmark) if window_bookmark else pendulum.utcnow()
+    window_start = pendulum.parse(state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]])
+    if previous_window_end == window_start:
+        raise apis.ExportFailed(
+            f"Export too large for smallest possible query window. Cannot subdivide any further."
+            f' ({stream["replication_key"]}: {window_start})'
+        ) from ex
 
-def sync_aqua_stream(client, state, stream, counter):
-    timed_out = False
+    half_day_range = (previous_window_end - window_start) // 2
+    current_window_end = previous_window_end - half_day_range
+    state["bookmarks"][stream["tap_stream_id"]]["current_window_end"] = current_window_end.strftime(
+        "%Y-%m-%dT%H:%M:%SZ"
+    )
+    singer.write_state(state)
+
+
+def sync_aqua_stream(client: Client, state: Dict, stream: Dict, counter):
+    """Performs sync for AQUA mode."""
     try:
         file_ids = state["bookmarks"][stream["tap_stream_id"]].get("file_ids")
         if not file_ids:
@@ -154,8 +166,7 @@ def sync_aqua_stream(client, state, stream, counter):
             state["bookmarks"][stream["tap_stream_id"]]["file_ids"] = file_ids
             singer.write_state(state)
 
-        window_end = state["bookmarks"][stream["tap_stream_id"]].pop("current_window_end", None)
-        if window_end:
+        if window_end := state["bookmarks"][stream["tap_stream_id"]].pop("current_window_end", None):
             # Save the window_end as the latest bookmark in case the window was empty
             state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]] = window_end
         return sync_file_ids(file_ids, client, state, stream, apis.Aqua, counter)
@@ -167,14 +178,17 @@ def sync_aqua_stream(client, state, stream, counter):
         LOGGER.info("Retrying timed out sync job...")
         return sync_aqua_stream(client, state, stream, counter)
 
-def handle_rest_timeout(ex, stream, state, current_window, start_pen):
+
+def handle_rest_timeout(ex, stream: Dict, state: Dict, current_window: int, start_pen: str) -> int:
     if stream.get("replication_key"):
         LOGGER.info("Export timed out, reducing query window and writing state.")
         new_window = current_window // 2
         if new_window == 0:
-            raise apis.ExportFailed("Export too large for smallest possible query window. " +
-                                    "Cannot subdivide any further. ({}: {})"
-                                    .format(stream["replication_key"], start_pen)) from ex
+            raise apis.ExportFailed(
+                f"Export too large for smallest possible query window. Cannot subdivide any further."
+                f' ({stream["replication_key"]}: {start_pen})'
+            ) from ex
+
         state["bookmarks"][stream["tap_stream_id"]]["window_length"] = new_window
         singer.write_state(state)
         return new_window
@@ -183,8 +197,16 @@ def handle_rest_timeout(ex, stream, state, current_window, start_pen):
     #     get hit (defensive coding might not be necessary above)
     return None
 
-def iterate_rest_query_window(client, state, stream, counter,
-                              start_pen, sync_started, window_length):
+
+def iterate_rest_query_window(
+    client: Client,
+    state: Dict,
+    stream: Dict,
+    counter,
+    start_pen,
+    sync_started,
+    window_length: int,
+):
     try:
         timed_out = False
         while start_pen < sync_started:
@@ -196,6 +218,7 @@ def iterate_rest_query_window(client, state, stream, counter,
             end_date = end_pen.strftime("%Y-%m-%d %H:%M:%S")
             job_id = apis.Rest.create_job(client, stream, start_date, end_date)
             file_ids = poll_job_until_done(job_id, client, apis.Rest)
+            LOGGER.info(f"file_ids for stream {stream['tap_stream_id']} are {file_ids}")
             counter = sync_file_ids(file_ids, client, state, stream, apis.Rest, counter)
             start_pen = end_pen
             window_length = MAX_EXPORT_DAYS * 86400
@@ -203,22 +226,17 @@ def iterate_rest_query_window(client, state, stream, counter,
             state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]] = end_date
             singer.write_state(state)
     except apis.ExportTimedOut as ex:
-        window_length = handle_rest_timeout(ex,
-                                            stream,
-                                            state,
-                                            window_length,
-                                            start_pen)
+        window_length = handle_rest_timeout(ex, stream, state, window_length, start_pen)
         timed_out = True
 
     if timed_out:
         LOGGER.info("Retrying timed out sync job...")
-        return iterate_rest_query_window(client, state, stream, counter,
-                                         start_pen, sync_started, window_length)
+        return iterate_rest_query_window(client, state, stream, counter, start_pen, sync_started, window_length)
     return counter
 
-def sync_rest_stream(client, state, stream, counter):
-    file_ids = state["bookmarks"][stream["tap_stream_id"]].get("file_ids")
-    if file_ids:
+
+def sync_rest_stream(client: Client, state: Dict, stream: Dict, counter):
+    if file_ids := state["bookmarks"][stream["tap_stream_id"]].get("file_ids"):
         counter = sync_file_ids(file_ids, client, state, stream, apis.Rest, counter)
 
     if stream.get("replication_key"):
@@ -227,8 +245,15 @@ def sync_rest_stream(client, state, stream, counter):
         sync_started = pendulum.utcnow()
         start_date = state["bookmarks"][stream["tap_stream_id"]][stream["replication_key"]]
         start_pen = pendulum.parse(start_date)
-        counter = iterate_rest_query_window(client, state, stream, counter,
-                                            start_pen, sync_started, window_length_in_seconds)
+        counter = iterate_rest_query_window(
+            client,
+            state,
+            stream,
+            counter,
+            start_pen,
+            sync_started,
+            window_length_in_seconds,
+        )
     else:
         job_id = apis.Rest.create_job(client, stream)
         file_ids = poll_job_until_done(job_id, client, apis.Rest)
@@ -236,9 +261,11 @@ def sync_rest_stream(client, state, stream, counter):
 
     return counter
 
-def sync_stream(client, state, stream, force_rest=False):
+
+def sync_stream(client: Client, state: Dict, stream: Dict):
+    """Starts the process for syncing the data for a given stream."""
     with singer.metrics.record_counter(stream["tap_stream_id"]) as counter:
-        if force_rest:
+        if client.is_rest:
             counter = sync_rest_stream(client, state, stream, counter)
         else:
             counter = sync_aqua_stream(client, state, stream, counter)
